@@ -2,7 +2,7 @@
  * libvirt-gobject-domain.c: libvirt glib integration
  *
  * Copyright (C) 2008 Daniel P. Berrange
- * Copyright (C) 2010 Red Hat
+ * Copyright (C) 2010-2011 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,10 +30,6 @@
 #include "libvirt-gobject/libvirt-gobject.h"
 #include "libvirt-gobject-compat.h"
 
-extern gboolean debugFlag;
-
-#define DEBUG(fmt, ...) do { if (G_UNLIKELY(debugFlag)) g_debug(fmt, ## __VA_ARGS__); } while (0)
-
 #define GVIR_DOMAIN_GET_PRIVATE(obj)                         \
         (G_TYPE_INSTANCE_GET_PRIVATE((obj), GVIR_TYPE_DOMAIN, GVirDomainPrivate))
 
@@ -49,6 +45,7 @@ G_DEFINE_TYPE(GVirDomain, gvir_domain, G_TYPE_OBJECT);
 enum {
     PROP_0,
     PROP_HANDLE,
+    PROP_PERSISTENT,
 };
 
 enum {
@@ -84,6 +81,10 @@ static void gvir_domain_get_property(GObject *object,
         g_value_set_boxed(value, priv->handle);
         break;
 
+    case PROP_PERSISTENT:
+        g_value_set_boolean(value, gvir_domain_get_persistent (conn));
+        break;
+
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     }
@@ -116,7 +117,7 @@ static void gvir_domain_finalize(GObject *object)
     GVirDomain *conn = GVIR_DOMAIN(object);
     GVirDomainPrivate *priv = conn->priv;
 
-    DEBUG("Finalize GVirDomain=%p", conn);
+    g_debug("Finalize GVirDomain=%p", conn);
 
     virDomainFree(priv->handle);
 
@@ -160,6 +161,17 @@ static void gvir_domain_class_init(GVirDomainClass *klass)
                                                        G_PARAM_STATIC_NICK |
                                                        G_PARAM_STATIC_BLURB));
 
+    g_object_class_install_property(object_class,
+                                    PROP_PERSISTENT,
+                                    g_param_spec_boolean("persistent",
+                                                         "Persistent",
+                                                         "If domain is persistent",
+                                                         TRUE,
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
+
     signals[VIR_STARTED] = g_signal_new("started",
                                         G_OBJECT_CLASS_TYPE(object_class),
                                         G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE |
@@ -215,13 +227,9 @@ static void gvir_domain_class_init(GVirDomainClass *klass)
 
 static void gvir_domain_init(GVirDomain *conn)
 {
-    GVirDomainPrivate *priv;
+    g_debug("Init GVirDomain=%p", conn);
 
-    DEBUG("Init GVirDomain=%p", conn);
-
-    priv = conn->priv = GVIR_DOMAIN_GET_PRIVATE(conn);
-
-    memset(priv, 0, sizeof(*priv));
+    conn->priv = GVIR_DOMAIN_GET_PRIVATE(conn);
 }
 
 typedef struct virDomain GVirDomainHandle;
@@ -287,10 +295,9 @@ gint gvir_domain_get_id(GVirDomain *dom,
     gint ret;
 
     if ((ret = virDomainGetID(priv->handle)) < 0) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to get ID for domain");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to get ID for domain");
     }
     return ret;
 }
@@ -313,10 +320,9 @@ gboolean gvir_domain_start(GVirDomain *dom,
     else
         ret = virDomainCreate(priv->handle);
     if (ret < 0) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to start domain");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to start domain");
         return FALSE;
     }
 
@@ -335,10 +341,9 @@ gboolean gvir_domain_resume(GVirDomain *dom,
     GVirDomainPrivate *priv = dom->priv;
 
     if (virDomainResume(priv->handle) < 0) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to resume domain");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to resume domain");
         return FALSE;
     }
 
@@ -362,10 +367,9 @@ gboolean gvir_domain_stop(GVirDomain *dom,
     else
         ret = virDomainDestroy(priv->handle);
     if (ret < 0) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to stop domain");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to stop domain");
         return FALSE;
     }
 
@@ -389,10 +393,9 @@ gboolean gvir_domain_delete(GVirDomain *dom,
     else
         ret = virDomainUndefine(priv->handle);
     if (ret < 0) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to delete domain");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to delete domain");
         return FALSE;
     }
 
@@ -411,10 +414,9 @@ gboolean gvir_domain_shutdown(GVirDomain *dom,
     GVirDomainPrivate *priv = dom->priv;
 
     if (virDomainShutdown(priv->handle) < 0) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to shutdown domain");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to shutdown domain");
         return FALSE;
     }
 
@@ -433,10 +435,9 @@ gboolean gvir_domain_reboot(GVirDomain *dom,
     GVirDomainPrivate *priv = dom->priv;
 
     if (virDomainReboot(priv->handle, flags) < 0) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to reboot domain");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to reboot domain");
         return FALSE;
     }
 
@@ -457,10 +458,9 @@ GVirConfigDomain *gvir_domain_get_config(GVirDomain *dom,
     gchar *xml;
 
     if (!(xml = virDomainGetXMLDesc(priv->handle, flags))) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to get domain XML config");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to get domain XML config");
         return NULL;
     }
 
@@ -496,7 +496,7 @@ gboolean gvir_domain_set_config(GVirDomain *domain,
     GVirDomainPrivate *priv = domain->priv;
 
     g_return_val_if_fail(GVIR_IS_DOMAIN (domain), FALSE);
-    g_return_val_if_fail(GVIR_IS_CONFIG_DOMAIN (conf), FALSE);
+    g_return_val_if_fail(GVIR_CONFIG_IS_DOMAIN (conf), FALSE);
     g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
 
     xml = gvir_config_object_to_xml(GVIR_CONFIG_OBJECT(conf));
@@ -504,10 +504,9 @@ gboolean gvir_domain_set_config(GVirDomain *domain,
     g_return_val_if_fail(xml != NULL, FALSE);
 
     if ((conn = virDomainGetConnect(priv->handle)) == NULL) {
-        if (err != NULL)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Failed to get domain connection");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Failed to get domain connection");
         g_free (xml);
 
         return FALSE;
@@ -517,11 +516,10 @@ gboolean gvir_domain_set_config(GVirDomain *domain,
     g_free (xml);
 
     if (handle == NULL) {
-        if (err != NULL)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Failed to set "
-                                          "domain configuration");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Failed to set "
+                               "domain configuration");
         return FALSE;
     }
 
@@ -529,11 +527,10 @@ gboolean gvir_domain_set_config(GVirDomain *domain,
     virDomainFree(handle);
 
     if (g_strcmp0 (uuid, priv->uuid) != 0) {
-        if (err != NULL)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Failed to set "
-                                          "domain configuration");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Failed to set "
+                               "domain configuration");
 
         return FALSE;
     }
@@ -554,10 +551,9 @@ GVirDomainInfo *gvir_domain_get_info(GVirDomain *dom,
     GVirDomainInfo *ret;
 
     if (virDomainGetInfo(priv->handle, &info) < 0) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to get domain info");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to get domain info");
         return NULL;
     }
 
@@ -599,10 +595,9 @@ gchar *gvir_domain_screenshot(GVirDomain *dom,
                                      st,
                                      monitor_id,
                                      flags))) {
-        if (err)
-            *err = gvir_error_new_literal(GVIR_DOMAIN_ERROR,
-                                          0,
-                                          "Unable to take a screenshot");
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to take a screenshot");
         goto end;
     }
 end:
@@ -610,3 +605,248 @@ end:
         virStreamFree(st);
     return mime;
 }
+
+
+/**
+ * gvir_domain_open_console:
+ * @dom: (transfer none): the domain
+ * @devname: (transfer none)(allow-none): the device name
+ * @stream: (transfer none): stream to use as output
+ * @flags: extra flags, currently unused
+ *
+ * Open a text console for the domain @dom, connecting it to the
+ * stream @stream. If @devname is NULL, the default console will
+ * be opened, otherwise @devname can be used to specify a non-default
+ * console device.
+ *
+ * Returns: TRUE if the console was opened, FALSE otherwise.
+ */
+gboolean gvir_domain_open_console(GVirDomain *dom,
+                                  GVirStream *stream,
+                                  const gchar *devname,
+                                  guint flags,
+                                  GError **err)
+{
+    GVirDomainPrivate *priv;
+    virStreamPtr st = NULL;
+    gboolean ret = FALSE;
+
+    g_return_val_if_fail(GVIR_IS_DOMAIN(dom), FALSE);
+    g_return_val_if_fail(GVIR_IS_STREAM(stream), FALSE);
+
+    priv = dom->priv;
+    g_object_get(stream, "handle", &st, NULL);
+
+    if (virDomainOpenConsole(priv->handle,
+                             devname,
+                             st,
+                             flags) < 0) {
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to open console");
+        goto cleanup;
+    }
+
+    ret = TRUE;
+cleanup:
+    if (st != NULL)
+        virStreamFree(st);
+    return ret;
+}
+
+
+/**
+ * gvir_domain_open_graphics:
+ * @dom: the domain
+ * @idx: the graphics index
+ * @fd: pre-opened socket pair
+ * @flags: extra flags, currently unused
+ *
+ * Open a connection to the local graphics display, connecting it to the
+ * socket pair file descriptor passed in as @fd.
+ *
+ * Returns: TRUE if the graphics connection was opened, FALSE otherwise.
+ */
+gboolean gvir_domain_open_graphics(GVirDomain *dom,
+                                   guint idx,
+                                   int fd,
+                                   unsigned int flags,
+                                   GError **err)
+{
+    GVirDomainPrivate *priv;
+    gboolean ret = FALSE;
+
+    g_return_val_if_fail(GVIR_IS_DOMAIN(dom), FALSE);
+
+    priv = dom->priv;
+
+    if (virDomainOpenGraphics(priv->handle,
+                              idx,
+                              fd,
+                              flags) < 0) {
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to open graphics");
+        goto cleanup;
+    }
+
+    ret = TRUE;
+cleanup:
+    return ret;
+}
+
+/**
+ * gir_domain_suspend:
+ * @dom: the domain to suspend
+ * @err: Place-holder for possible errors
+ *
+ * Suspends an active domain, the process is frozen without further access to
+ * CPU resources and I/O but the memory used by the domain at the hypervisor
+ * level will stay allocated. Use gvir_domain_resume() to reactivate the domain.
+ *
+ * Returns: TRUE if domain was suspended successfully, FALSE otherwise.
+ */
+gboolean gvir_domain_suspend (GVirDomain *dom,
+                              GError **err)
+{
+    gboolean ret = FALSE;
+
+    g_return_val_if_fail(GVIR_IS_DOMAIN(dom), FALSE);
+
+    if (virDomainSuspend(dom->priv->handle) < 0) {
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to suspend domain");
+        goto cleanup;
+    }
+
+    ret = TRUE;
+cleanup:
+    return ret;
+}
+
+/**
+ * gvir_domain_save:
+ * @dom: the domain to save and suspend
+ * @flags: extra flags, currently unused
+ * @err: Place-holder for possible errors
+ *
+ * Just like #gvir_domain_suspend but also saves the state of the domain on disk
+ * and therefore makes it possible to restore the domain to its previous state
+ * even after shutdown/reboot of host machine.
+ *
+ * Returns: TRUE if domain was saved and suspended successfully, FALSE otherwise.
+ */
+gboolean gvir_domain_save (GVirDomain *dom,
+                           unsigned int flags,
+                           GError **err)
+{
+    g_return_val_if_fail(GVIR_IS_DOMAIN(dom), FALSE);
+
+    if (virDomainManagedSave(dom->priv->handle, flags) < 0) {
+        gvir_set_error_literal(err, GVIR_DOMAIN_ERROR,
+                               0,
+                               "Unable to save and suspend domain");
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+typedef struct {
+    guint flags;
+} DomainSaveData;
+
+static void domain_save_data_free(DomainSaveData *data)
+{
+    g_slice_free (DomainSaveData, data);
+}
+
+static void
+gvir_domain_save_helper(GSimpleAsyncResult *res,
+                        GObject *object,
+                        GCancellable *cancellable G_GNUC_UNUSED)
+{
+    GVirDomain *dom = GVIR_DOMAIN(object);
+    DomainSaveData *data;
+    GError *err = NULL;
+
+    data = g_simple_async_result_get_op_res_gpointer (res);
+
+    if (!gvir_domain_save(dom, data->flags, &err))
+        g_simple_async_result_take_error(res, err);
+}
+
+/**
+ * gir_domain_save_async:
+ * @dom: the domain to save and suspend
+ * @flags: extra flags, currently unused
+ * @cancellable: (allow-none)(transfer none): cancellation object
+ * @callback: (scope async): completion callback
+ * @user_data: (closure): opaque data for callback
+ *
+ * Asynchronous variant of #gvir_domain_save.
+ */
+void gvir_domain_save_async (GVirDomain *dom,
+                             unsigned int flags,
+                             GCancellable *cancellable,
+                             GAsyncReadyCallback callback,
+                             gpointer user_data)
+{
+    GSimpleAsyncResult *res;
+    DomainSaveData *data;
+
+    g_return_if_fail(GVIR_IS_DOMAIN(dom));
+
+    data = g_slice_new0(DomainSaveData);
+    data->flags = flags;
+
+    res = g_simple_async_result_new(G_OBJECT(dom),
+                                    callback,
+                                    user_data,
+                                    gvir_domain_save_async);
+    g_simple_async_result_set_op_res_gpointer (res, data, (GDestroyNotify) domain_save_data_free);
+    g_simple_async_result_run_in_thread(res,
+                                        gvir_domain_save_helper,
+                                        G_PRIORITY_DEFAULT,
+                                        cancellable);
+    g_object_unref(res);
+}
+
+/**
+ * gir_domain_save_finish:
+ * @dom: the domain to save and suspend
+ * @result: (transfer none): async method result
+ * @err: Place-holder for possible errors
+ *
+ * Finishes the operation started by #gvir_domain_save_async.
+ *
+ * Returns: TRUE if domain was saved and suspended successfully, FALSE otherwise.
+ */
+gboolean gvir_domain_save_finish (GVirDomain *dom,
+                                  GAsyncResult *result,
+                                  GError **err)
+{
+    g_return_val_if_fail(GVIR_IS_DOMAIN(dom), FALSE);
+    g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(dom),
+                                                        gvir_domain_save_async),
+                         FALSE);
+
+    if (g_simple_async_result_propagate_error(G_SIMPLE_ASYNC_RESULT(result), err))
+        return FALSE;
+
+    return TRUE;
+}
+
+/**
+ * gvir_domain_get_persistent:
+ * @dom: the domain
+ *
+ * Returns: TRUE if domain is persistent, FALSE otherwise.
+ */
+gboolean gvir_domain_get_persistent(GVirDomain *dom)
+{
+    g_return_val_if_fail(GVIR_IS_DOMAIN(dom), FALSE);
+
+    return virDomainIsPersistent(dom->priv->handle);
+}
